import { useEffect, useState } from 'react'

import { SpritesData } from './types'

const endpoint = process.env.REACT_APP_SPRITES_ENDPOINT

export const useSprites = (id: string | undefined) => {
  const [spritesData, setSpritesData] = useState<SpritesData>()
  useEffect(() => {
    fetch(`${endpoint}/sprites/${id}`)
      .then((response) => response.json())
      .then((json) => {
        const data = json as SpritesData
        setSpritesData({ ...data })
        const spritesB64: { url: string; dataUrl: string }[] = []
        data.sprites.map((i) => {
          fetchAsBlob(`${endpoint}${i}`)
            .then(blobToBase64)
            .then((b64) => {
              if (b64) spritesB64.push({ url: i.toString(), dataUrl: b64.toString() })
              setSpritesData({
                ...data,
                spritesLoaded: spritesB64.sort((a, b) => (a.url > b.url ? 1 : b.url > a.url ? -1 : 0)),
              })
            })
        })
      })
  }, [id])

  if (spritesData) {
    return spritesData
  }
  return undefined
}

const fetchAsBlob = (url: string) => fetch(url).then((response) => response.blob())

const blobToBase64 = (blob: Blob) =>
  new Promise<string | ArrayBuffer | null>((resolve, reject) => {
    const reader = new FileReader()
    reader.onerror = reject
    reader.onload = () => {
      resolve(reader.result)
    }
    reader.readAsDataURL(blob)
  })
