import videojs from 'video.js'

import { SpritesData } from '@/providers/assets/types'

export const Component = videojs.getComponent('Component')
export const MouseTimeDisplay = videojs.getComponent('MouseTimeDisplay')

export class MouseTimeAndThumbnailDisplay extends MouseTimeDisplay {
  async update(seekBarRect: ClientRect, seekBarPoint: number) {
    super.update(seekBarRect, seekBarPoint)
    const time = Math.round(seekBarPoint * this.player_.duration())
    this.updateThumbnail(time)
  }

  createEl() {
    const thumb = Component.prototype.createEl('div', { className: 'vjs-mouse-thumbnail' }) as HTMLDivElement
    const element = super.createEl()
    element.appendChild(thumb)
    return element
  }

  updateThumbnail(time: number) {
    const options = this.player_.options_.plugins?.mouseTimeThumbnails as SpritesData
    if (!options) {
      return
    }
    const rowSize = options.metadata.spriteRowSize
    const colSize = options.metadata.spriteMaxLines
    const bgWidth = options.metadata.thumbWidth * rowSize
    const bgHeight = options.metadata.thumbHeight * colSize
    const spriteIndex = Math.floor(time / (rowSize * colSize))
    const spriteDataUrl = options.spritesLoaded[spriteIndex].dataUrl
    const spriteTime = time - rowSize * colSize * spriteIndex
    const thumbWidth = bgWidth / rowSize
    const thumbHeight = bgHeight / colSize
    const currentRow = Math.floor(spriteTime / rowSize)
    const posY = currentRow * thumbHeight
    const currentCol = Math.floor(spriteTime % rowSize)
    const posX = currentCol * thumbWidth
    const thumbnailAttribute = '.vjs-mouse-thumbnail'
    const bgStyle = `
     width: ${thumbWidth}px; 
     height: ${thumbHeight}px; 
     display: block; 
     top: -${thumbHeight + 60}px;
     left: -${thumbWidth / 2}px;
     background-size: ${bgWidth}px ${bgHeight}px; 
     background-image: url('${spriteDataUrl}'); 
     background-position: ${posX}px -${posY}px;`
    //console.log(`sprite[${spriteIndex}] time[${time}] row[${currentRow}] col[${currentCol}] posX[${posX}] posY[${posY}]`)
    this.$(thumbnailAttribute).setAttribute('style', bgStyle)
  }
}
